/* input.c - part of clavier
 *
 * Copyright (C) 1998 Simon Kgedal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License in the file COPYING for more details.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "midievents.h"
#include "glib_1_0_compat.h"

/* from gui.c */
extern MidiHandle *midihandle;

static int input_fd;

/* this callback is called by gdk when there's something to read from input
 */

static void
input_read (gpointer data, gint source, GdkInputCondition condition)
{
  unsigned char c;

  read (input_fd, &c, 1);

  /*  printf ("condition: %i\tsource: %i\tdata: %i\t(input_fd: %i)\n",
   *  condition, source, (int)data, input_fd);
   */

  /*  printf ("read: 0x%x\t%i\n", (int)c, (int)c);
   */

  midi_add_byte (midihandle, c);
}

/* initialize input
 */

gint
input_init (char *filename)
{
  /* read from standard input? 
   */

  if (!strcmp (filename, "-"))
    {
      input_fd = 0;
    }

  /* no? create fifo then
   */

  else
    {
      if (mkfifo (filename, 0777) != 0)
	{
	  fprintf(stderr, "%s: %s: %s\n",
		  g_get_prgname (), filename, g_strerror (errno));
	  return 1;
	}

      input_fd = open (filename, O_RDONLY);
      if (input_fd == -1)
	{
	  fprintf(stderr, "%s: %s: %s\n",
		  g_get_prgname (), filename, g_strerror (errno));
	  return 1;
	}
    }

  gdk_input_add (input_fd,
		 GDK_INPUT_READ,
		 input_read,
		 NULL);

  return 0;
}

/* close input
 */

gint
input_exit (void)
{
  if (close (input_fd) == -1)
    {
      fprintf (stderr, "%s: %s\n", 
	       g_get_prgname (), g_strerror (errno));
    }

  return 0;
}

