/* out.c - part of clavier
 * Copyright (C) 1997,1998 Simon Kgedal
 *
 * functions that have to do with the output
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License in the file COPYING for more details.
 *
 * file created Apr 29 1997
 */

#include <config.h>
#include <stdio.h>
#include <glib.h>
#include "glib_1_0_compat.h"

/* these are needed for `open' */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <unistd.h>		/* for `write' */

#include <errno.h>
#include "midi.h"
#include "out.h"

Output *
output_new(void)
{
  return (Output*) g_new0 (Output, 1);
}

void
output_free (Output * output)
{
  g_free (output);
}

void
output_write (Output *output, const char *buf, int n)
{
  if (output->is_active)
    {
      /*      fwrite (buf, sizeof(char), n, output->outfile);*/
      write(output->outfd, buf, n);
    }
}

int
output_open(Output *output, const char *filename)
{
  /* ERRFIX */

  if (!strcmp(filename, "-"))
    {
      output->outfd = 1;
      output->is_active = TRUE;
    }
  else
    {
      output->outfd = open (filename, O_WRONLY);
      if (output->outfd == -1)
	{
	  fprintf (stderr, "%s: `%s': %s\n",
		   g_get_prgname (), filename, g_strerror (errno));
	  return (1);
	}
      output->is_active = TRUE;
    }

  return(0);
}

void
output_close(Output *output)
{
  /*  fclose(output->outfile);*/
  close (output->outfd);
  output->is_active = FALSE;
}

/************************
 * midi aware functions *
 ************************/

void
output_noteon(Output *output, int channel, int note, int vel)
{
  char midimess[3];

  midimess[0] = MIDI_NOTE_ON | channel;
  midimess[1] = note;
  midimess[2] = vel;
  output_write (output, midimess, 3);
}

void
output_noteoff(Output *output, int channel, int note)
{
  char midimess[3];

  midimess[0] = MIDI_NOTE_OFF | channel;
  midimess[1] = note;
  midimess[2] = 0;
  output_write (output, midimess, 3);
}

void
output_changeprog (Output *output, int channel, int program)
{
  char midimess[2];

  midimess[0] = MIDI_PROG_CHANGE | channel;
  midimess[1] = program;
  output_write (output, midimess, 2);
}

/* i should maybe generalize a output_control first... */

void
output_all_notes_off (Output *output, int channel)
{
  char midimess[3];

  midimess[0] = MIDI_CTRL_CHANGE | channel;
  midimess[1] = MIDIC_ALL_NOTES_OFF;
  midimess[2] = 0;
  output_write (output, midimess, 3);
}
