/* progchange.c - part of clavier
 *
 * Copyright (C) 1998 Simon Kgedal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License in the file COPYING for more details.
 */

#include "out.h"
#include <gtk/gtk.h>

/* from main.c */
extern Output *output;

typedef struct _ProgChangeInfo ProgChangeInfo;

struct _ProgChangeInfo
{
  GtkWidget *channel_spinner;
  GtkWidget *prog_spinner;
};

static void
send_progchange (GtkWidget *button, ProgChangeInfo *info)
{
  gint channel, prog;

  g_return_if_fail (info != NULL);

  channel = gtk_spin_button_get_value_as_int
    (GTK_SPIN_BUTTON (info->channel_spinner)) - 1;
  prog = gtk_spin_button_get_value_as_int 
    (GTK_SPIN_BUTTON (info->prog_spinner)) - 1;

  output_changeprog (output, channel, prog);
}

void
progchange_show (GtkWidget *channelspinner)
{
  GtkWidget *window;
  GtkWidget *hbox;
  GtkWidget *spinner;
  GtkWidget *button;
  GtkObject *adjustment;

  /* TODO: this never gets freed. fix. */
  ProgChangeInfo *info = g_new0 (ProgChangeInfo, 1);

  info->channel_spinner = channelspinner;

  /* main window */

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Change Program");

  gtk_container_border_width (GTK_CONTAINER (window), 10);

  /* hbox */

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), hbox);
  
  /* spinner */

  adjustment = gtk_adjustment_new (1.0, 1.0, 128.0, 1.0, 5.0, 0.0);

  spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1.0, 0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spinner),
				     GTK_UPDATE_ALWAYS);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinner), TRUE);

  gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 5);
  gtk_widget_show (spinner);

  info->prog_spinner = spinner;

  /* button */

  button = gtk_button_new_with_label ("Send");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (send_progchange), 
		      (gpointer) info);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 5);
  gtk_widget_show (button);

  gtk_widget_show (hbox);
  gtk_widget_show (window);
}
