/* util.c - part of clavier
 * Copyright (C) 1997,1998 Simon Kgedal
 * timeval_substract is from the GNU libc infofile, and is
 * Copyright of Free Software Foundation, under the GPL.
 *
 * small utility functions
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License in the file COPYING for more details.
 *
 * file created Apr 29 1997
 */

#include <stdlib.h>
#include <string.h>
#include "util.h"

/* converts a note name as they look in the config file to a number
 * where c0 = 0
 *
 * name:	note name; c0, c#0, d0... c1, c#1, d1, d#1....
 * note:	the returned note number
 *
 * returns:	TRUE if Ok, FALSE if it can't be parsed,
 */

gboolean
notenumber(char *name, int *note)
{
  static char *notes[] = { "c", "c#", "d", "d#", "e", "f", "f#", "g",
			   "g#", "a", "a#", "b" };
 
  int i, found_note = -1;

  for(i = 0; i < 12; ++i)
    {
      if(!strncmp(name, notes[i], strlen(notes[i])))
        found_note = i;
    }

  if(found_note != -1)
    {
      *note = found_note + atoi(name + strlen(notes[found_note])) * 12;
      return(TRUE);
    }
  else
    return(FALSE);
}

/* creates a string with the name of a note
 *
 * note:         the number of the note, 0 being c0.
 *
 * returns:     a string, which should be freed after usage
 *
 */

char *
notename(int note)
{
  static char *notes[] = { "c", "c#", "d", "d#", "e", "f", "f#", "g",
			   "g#", "a", "a#", "b" };
  char *s;

  s = g_malloc(6);
  sprintf(s, "%s%i", notes[note % 12], note/12);
  return(s);
}
      
/* Reads one complete line from the file FILE, allocates space
 * for it and puts in `*buf', NOT including the ending '\n'.  
 * The length of the line is put in `line_length'.
 * *buf should be freed with g_free ().
 *
 * Returns: TRUE on success, FALSE if an error occured or there's
 * 	    nothing more to read.
 *
 * TODO: the implementation of `getline' in glibc is probably
 *	 better, copy/use that
 */ 

gboolean
my_getline(char **buf, int *line_length, FILE *file)
{
  static int chunk_size = 256;

  char c;
  int bufsize = chunk_size;
  int i = 0;

  *buf = g_malloc (bufsize);
  
  while (1)
    {
      if (fread (&c, 1, 1, file) != 1)
	break;
      if (c == '\n')
	{
	  (*buf)[i] = '\0';
	  break;
	}

      (*buf)[i] = c;
      ++i;
      if (i == bufsize)
	{
	  bufsize += chunk_size;
	  *buf = g_realloc (*buf, bufsize);
	}
    }

  if (i == 0 && feof (file))
    return FALSE;

  if (ferror (file))
    return FALSE;

  *line_length = i;
  return TRUE;
}
